/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package Celbercor;

import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;

/** This class is the authentication server.  It has a <tt>main()</tt>
 * function which creates and runs the server.  */

public class AuthenticationServer
  extends ServerBase
{

  /**
   * The default port to listen to.
   */

  public static final int DEFAULT_PORT = 10203;


  /**
   * The default host the server should run on.
   */

  public static final String DEFAULT_HOST;


  static
    {
      InetAddress a = null;
      try
	{
	  a = InetAddress.getLocalHost();
	}
      catch( UnknownHostException ex )
	{
	  ex.printStackTrace();
	}
      
      if( a == null )
	{
	  DEFAULT_HOST = "localhost";
	}
      else
	{
	  DEFAULT_HOST = a.getHostName();
	}
    }

  /** Get a request object from someone (a
   * <tt>AuthenticationRequestMessage</tt>), and attempt to formulate
   * a reply (a <tt>AuthenticationReplyMessage</tt>).  If the request
   * is invalid, send <tt>null</tt> as a reply.  */

  public void handleOneConnection( Socket s )
    {
      try
	{
	  AuthenticationRequestMessage request;
	  AuthenticationReplyMessage reply;
	  SecretKey endClientKey;
	  SecretKey serviceKey;
	  SealedObject sealedReply = null;

	  System.out.println( "Connection established, reading request" );
	  ObjectInput in = new ObjectInputStream( s.getInputStream() );
      
	  request = (AuthenticationRequestMessage)in.readObject();

	  endClientKey = getPasswords().get( request.getEndClientName() );
	  serviceKey = getPasswords().get( request.getServiceName() );

	  if( endClientKey == null )
	    {
	      System.out.println( "Unknown end client: "
				  + request.getEndClientName() );
	    }
	  else if( serviceKey == null )
	    {
	      System.out.println( "Unknown service: "
				  + request.getServiceName() );

	    }
	  else
	    {
	      System.out.println( "Request was valid, formulating reply" );

	      // Do know who this is...
	      Cipher toEndClient = 
		EncryptionStandard.getInstance().createCipher();
	      toEndClient.init( Cipher.ENCRYPT_MODE, endClientKey );

	      Cipher toService = 
		EncryptionStandard.getInstance().createCipher();
	      toService.init( Cipher.ENCRYPT_MODE, serviceKey );

	      // Need to generate a session key:
	      SecretKey sessionKey = 
		EncryptionStandard.getInstance().createSessionKey();
	      Ticket t;
	      t = new Ticket( request.getEndClientName(),
			      s.getInetAddress(),
			      request.getServiceName(),
			      sessionKey,
			      System.currentTimeMillis() );

	      SealedObject st = new SealedObject( t, toService );
	      
	      // We don't want to reply with
	      // { Ticket, { (same)Ticket } K_AS_TS } K_AS_C
	      // because someone might be able to crack K_AS_TS if
	      // they asked for a bunch of tickets and made a whole
	      // set of (t, {t} K_AS_TS).  So, we send
	      // { { Ticket } K_AS_TS, (almost the same)Ticket } K_AS_C
	      // by changing the time stamp a small amount.
	      t = new Ticket( request.getEndClientName(),
			      s.getInetAddress(),
			      request.getServiceName(),
			      sessionKey,
			      System.currentTimeMillis() + 42 );
	      reply = new AuthenticationReplyMessage( t, st );

	      sealedReply = new SealedObject( reply, toEndClient );

	    }
	  System.out.println( "Sending reply" );
	  ObjectOutput out = new ObjectOutputStream( s.getOutputStream() );
	  out.writeObject( sealedReply );
	}
      catch( Exception ex )
	{
	  System.out.println( "Request failed:" );
	  ex.printStackTrace();
	}
    }

  /**
   * Starts an authentication server.  It expects arguments as follows:
   *
   * <pre>
   * java Celbercor.AuthenticationServer <i>passwordFile</i> [<i>port</i>]
   * </pre>
   *
   * where <i>passwordFile</i> is the name of the password file, and
   * <i>port</i> is the port to listen to (default value is
   * <tt>AuthenticationServer.DEFAULT_PORT</tt>).  The password file
   * must contain a serialized <tt>PasswordKeyList</tt>, which can be
   * constructed with the <tt>main()</tt> function in class
   * <tt>PasswordKeyList</tt>.  The file should contain keys for all
   * users and the ticket server.
   *
   * @see Celbercor.PasswordKeyList
   * */

  public static void main( String args[] )
    {
      String passwordFile;
      int port = DEFAULT_PORT;
      
      if( args.length < 1 )
	{
	  System.out.println( "Arguments: passwordFile [port]" );
	  System.exit(1);
	}

      passwordFile = args[0];

      if( args.length >= 2 )
	{
	  port = Integer.parseInt( args[1] );
	}

      try
	{
	  EncryptionStandard.getInstance().initializeEncryptionLibrary();

	  AuthenticationServer s = new AuthenticationServer();
	  s.readPasswords( passwordFile );
	  s.open( DEFAULT_PORT );
	  System.out.println( "Read passwords from " + passwordFile );
	  System.out.println( "Listening for connections on port " + port );
	  s.run();
	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	}
    }
}
