/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package Celbercor;

import java.util.*;
import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;

/** This is a debugging program that contacts the authentication
 * server and the ticket server.  It prints out a lot of debugging
 * information as it works.  */

public class SampleClient
{
  /**
   * To use this main function:
   *
   * <pre>
   * java Celbercor.SampleClient <i>endClientName</i> <i>serviceName</i>
   * </pre>
   *
   * where <tt><i>endClientName</i></tt> is your login name and
   * <tt><i>serviceName</i></tt> is the service you want it to
   * contact.  It will authenticate you with the authenticaion server
   * and get a ticket from the ticket server, but will not attempt to
   * contact the service.
   *
   * <p> To get the name and port of the authentication and ticket
   * servers, it looks at the standard system properties (or the
   * defaults if the resource is not set).  
   *
   * @see <a href="package-summary.html#properties">Properties</a>
   */

  public static void main( String args[] )
    {
      String endClientName;
      String serviceName;
      String serviceHost;
      int servicePort;

      String host;
      int port;

      if( args.length != 2 )
	{
	  System.out.println( 
	    "Arguments: endClientName serviceName" );
	  System.exit(1);
	}

      endClientName = args[0];
      serviceName = args[1];
      serviceHost = args[2];
      servicePort = Integer.parseInt( args[3] );

      host = System.getProperty( "celbercor.authentication.host",
				 AuthenticationServer.DEFAULT_HOST );
      port = Integer.getInteger( "celbercor.authentication.port",
				 AuthenticationServer.DEFAULT_PORT ).intValue();

      try
	{
	  EncryptionStandard.getInstance().initializeEncryptionLibrary();

	  SealedObject st = 
	    Utilities.contactAuthenticationServer( host, port, endClientName );

	  System.out.println( "Enter password:" );
	  char password[] = 
	    EncryptionStandard.getInstance().readPassword( System.in );

	  SecretKey key = 
	    EncryptionStandard.getInstance().convertPassword( password );
	  Arrays.fill( password, '*' );

	  AuthenticationReplyMessage sreply = null;
	  Ticket tst = null;
	  boolean suceeded = true;

	  try
	    {
	      System.out.println( "Attempting to decode ticket" );
	      sreply  = (AuthenticationReplyMessage)st.getObject( key );
	      suceeded = true;
	    }
	  catch( InvalidKeyException ex )
	    {
	      suceeded = false;
	    }
		
	  if( suceeded )
	    {
	      tst = sreply.getTicket();

	      // Have to check that this worked, too.
	      if( tst.getEndClientName().equals( endClientName )
		  && tst.getServiceName().equals( serviceName ) )
		{
		  System.out.println( "Decoded ticket:" );
		  System.out.println( "End client: " 
				      + tst.getEndClientName() );
		  System.out.println( "My address: " 
				      + tst.getEndClientAddress() );
		  System.out.println( "Service name: " 
				      + tst.getServiceName() );
		  System.out.println( "Session key: " 
				      + tst.getSessionKey() );
		  System.out.println( "Time stamp: "
				      + (new Date(tst.getTime())) );
		}
	    }
	  else
	    {
	      System.out.println( "Password incorrect" );
	      System.exit(3);
	    }

	  // Now try to get a session ticket from the ticket server
	  System.out.println( "Attempting to contact ticket server" );

	  host = System.getProperty( "celbercor.ticket.host",
				     TicketServer.DEFAULT_HOST );
	  port =
	    Integer.getInteger( "celbercor.ticket.port",
				TicketServer.DEFAULT_PORT ).intValue();

	  TicketReplyMessage treply =
	    Utilities.contactTicketServer( host, port, 
					   serviceName,
					   tst, sreply.getSealedTicket() );

	  System.out.println( "Decoding reply" );
	  if( treply == null )
	    {
	      System.out.println( "Reply was null" );
	    }
	  else
	    {
	      System.out.println( "Reply was not null" );
	    }

	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	  System.exit(2);
	}

    }

}
