/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package Celbercor;

import java.net.InetAddress;
import java.util.*;
import javax.crypto.*;

/** (Steps 2-5) The authentication server and the ticket server create
 * <tt>Tickets</tt> and send them to end clients and service servers.
 * They contain session keys and time stamps that allow servers to
 * verify that the client that sent it is who it claims to be. */

public class Ticket
  implements java.io.Serializable
{
  public Ticket( String endClientName, InetAddress endClientAddress,
		 String serviceName, SecretKey sessionKey,
		 long time )
    {
      myEndClientName = endClientName;
      myEndClientAddress = endClientAddress;
      myServiceName = serviceName;
      mySessionKey = sessionKey;
      myTime = time;
    }

  public String getEndClientName()
    {
      return myEndClientName;
    }

  public InetAddress getEndClientAddress()
    {
      return myEndClientAddress;
    }

  public String getServiceName()
    {
      return myServiceName;
    }

  public SecretKey getSessionKey()
    {
      return mySessionKey;
    }

  public long getTime()
    {
      return myTime;
    }

  /** How long in milliseconds a ticket remains valid. (Currently one
   * hour.  In Kerberos, the expiration time may vary and is part of
   * the ticket.  For simplicity, Celbercor just uses a constant.)  */

  public static final long TIME_GAP = 60*60*1000; // this is an hour


  /** A function that compares two times to see if they are separated
   * by more than <tt>TIME_GAP</tt>ms.
   * 
   * @return <tt>null</tt> if the times are close enough, an error
   * message if they are too far apart.  */

  public static String checkTimes( long t1, long t2 )
    {
      String msg = null;

      if( Math.abs(t1-t2) > TIME_GAP )
	{
	  msg = "Bad times: "
	    + (new Date(t1))
	    + " more than " + TIME_GAP 
	    + " ms away from "
	    + (new Date(t2));
	}
      return msg;
    }

  /**
   * Check that this ticket is consistent with the given information.
   * @return null if everything is consistent, a string explaining
   * what is wrong if something is inconsistent.  */

  public String checkConsistency( String endClientName,
				  InetAddress endClientAddress,
				  String serviceName,
				  long time )
    {
      String msg;

      if( !getEndClientName().equals(endClientName) )
	{
	  msg = "End client names don't match (1): " + getEndClientName()
	    + " should be " + endClientName;
	}
      else if( !getEndClientAddress().equals(endClientAddress) )
	{
	  msg = "End client address doesn't match: "
	    + getEndClientAddress()
	    + " should be "
	    + endClientAddress;
	}
      else if( !getServiceName().equals(serviceName) )
	{
	  msg = "Ticket service name is wrong: "
	    + getServiceName()
	    + " should be "
	    + serviceName;
	}
      else
	{
	  msg = checkTimes(getTime(),time);
	}
      return msg;
    }


  /** Check that this ticket is consistent with the given
   *<tt>SessionAuthenticator</tt>, <tt>TicketRequestMessage</tt>,
   *<tt>InetAddress</tt>, and system time.
   *
   * @return null if everything is consistent, a string explaining
   * what is wrong if something is inconsistent.  */

  public String checkConsistency( SessionAuthenticator a,
				  String serviceName,
				  InetAddress endClientAddress,
				  long time )
    {
      String msg = null;

      if( !endClientAddress.equals(a.getEndClientAddress()) )
	{
	  msg = "End client addresses don't match (2): "
	    + endClientAddress
	    + " and "
	    + a.getEndClientAddress();
	}
      if( msg == null )
	{
	  msg = checkTimes( a.getTime(), time );
	}
      if( msg == null )
	{
	  msg = checkConsistency( a.getEndClientName(),
				  endClientAddress,
				  serviceName,
				  time );
	}

      return msg;
    }

  private String myEndClientName;
  private InetAddress myEndClientAddress;
  private String myServiceName;
  private SecretKey mySessionKey;
  private long myTime;
}
