/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package Celbercor;

import java.util.*;
import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;

/** This class implements the ticket granting service for Celbercor,
 * and is known in short as the ticket server.  It loads a table of
 * password keys from a file containing a serialized
 * <tt>PasswordKeyList</tt>.  There must be keys there for all user
 * and all services that they might use.  
 *
 * <p> The <tt>main()</tt> function runs a <tt>TicketServer</tt>.
 *
 * @see Celbercor.PasswordKeyList */

public class TicketServer
  extends ServerBase
{
  /**
   * The default port the server should listen to. (Currently 10204.)
   */

  public static final int DEFAULT_PORT = 10204;

  /** The name of the ticket server.  This name is encoded in various
   * messages, and is used to look up the ticket server's password
   * key, so there must be an entry for it in the password file that
   * the authentication server reads.  (Current value is
   * "celbercor/ticket-server".)  */

  public static final String NAME = "celbercor/ticket-server";

  /**
   * The default host the ticket server runs on.
   */

  public static final String DEFAULT_HOST;

  static
    {
      InetAddress a = null;
      try
	{
	  a = InetAddress.getLocalHost();
	}
      catch( UnknownHostException ex )
	{
	  ex.printStackTrace();
	}
      
      if( a == null )
	{
	  DEFAULT_HOST = "localhost";
	}
      else
	{
	  DEFAULT_HOST = a.getHostName();
	}
    }

  public TicketServer( SecretKey key )
    {
      myKey = key;
    }

  /** Get a <tt>TicketReqestMessage</tt> (sealed under the ticket
   * server's password), attempt to decode it, and formulate a reply
   * in the form of a <tt>TicketReplyMessage</tt> (sealed under the
   * end-client's password).  If the request is invalid, send
   * <tt>null</tt> as the reply. */

  public void handleOneConnection( Socket s )
    {
      try
	{

	  System.out.println( "Connection established" );
	  TicketRequestMessage request = null;
	  Ticket t = null;
	  SessionAuthenticator a = null;
	  SealedObject sealedReply = null;

	  System.out.println( "Decrypting request" );
	  ObjectInput in = new ObjectInputStream( s.getInputStream() );
	  request = (TicketRequestMessage)in.readObject();
	  t = (Ticket)request.getSealedTicket().getObject( myKey );
	  
	  // This is the end client/ticket server session key.
	  // (ec/ts)
	  SecretKey ec_ts_key = t.getSessionKey();
	  a = (SessionAuthenticator)
	    request.getSealedAuthenticator().getObject( ec_ts_key );
	  String msg = null;

	  System.out.println( "Verifying request" );

	  msg = t.checkConsistency( a, NAME, s.getInetAddress(),
				    System.currentTimeMillis() );

	  if( msg != null )
	    {
	      // Oops, messages weren't right somehow:
	      System.out.println( "Request is inconsistent" );
	      System.out.println( msg );
	    }
	  else
	    {
	      // Messages were consistent
	      System.out.println( "Request is consistent" );

	      SecretKey endClientKey = 
		getPasswords().get( t.getEndClientName() );

	      // (Note that t.getServiceName() is the name of the
	      // ticket server.
	      SecretKey serviceKey =
		getPasswords().get( request.getServiceName() );

	      if( endClientKey == null )
		{
		  msg = "Unknown end client: " + t.getEndClientName();
		}
	      else if( serviceKey == null )
		{
		  msg = "Unknown service: "
		    + request.getServiceName();
		}

	      if( msg != null )
		{
		  // Oops, service or client is unknown
		  System.out.println( msg );
		}
	      else
		{
		  System.out.println( "Formulating reply" );

		  TicketReplyMessage reply;
		  SecretKey ec_s_key = 
		    EncryptionStandard.getInstance().createSessionKey();

		  Ticket replyTicket;
		  replyTicket = new Ticket( t.getEndClientName(),
					    t.getEndClientAddress(),
					    request.getServiceName(),
					    ec_s_key,
					    System.currentTimeMillis());

		  // Now we seal the new ticket so that only the
		  // service can read it.
		  Cipher toService = 
		    EncryptionStandard.getInstance().createCipher();
		  toService.init( Cipher.ENCRYPT_MODE, serviceKey );
		  SealedObject sealedReplyTicket = 
		    new SealedObject( replyTicket, toService );
		  
		  // As in AuthenticationServer, we change the time
		  // stamp slightly to keep the sealed and unsealed
		  // tickets from yielding any insight to our
		  // K_C_TS key.
		  replyTicket = new Ticket( t.getEndClientName(),
					    t.getEndClientAddress(),
					    request.getServiceName(),
					    ec_s_key,
					    System.currentTimeMillis() + 42);
		  reply = new TicketReplyMessage( replyTicket,
						  sealedReplyTicket );
		  Cipher toEndClient = 
		    EncryptionStandard.getInstance().createCipher();
		  toEndClient.init( Cipher.ENCRYPT_MODE, ec_ts_key );

		  // The big moment:
		  sealedReply = new SealedObject( reply, toEndClient );
		}
	    }

	  System.out.println( "Sending reply" );
	  ObjectOutput out = new ObjectOutputStream( s.getOutputStream() );
	  out.writeObject( sealedReply );
	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	}
    }

  private SecretKey myKey;


  /**
   * Starts a ticket server.  It expects arguments as follows:
   *
   * <pre>
   * java Celbercor.TicketServer <i>passwordFile</i> [<i>port</i>]
   * </pre>
   *
   * The password file must contain a serialized
   * <tt>PasswordKeyList</tt> with entries for all users and services.  */

  public static void main( String args[] )
    {
      String passwordFile;
      int port = DEFAULT_PORT;
      
      if( args.length < 1 )
	{
	  System.out.println( "Arguments: passwordFile [port]" );
	  System.exit(1);
	}

      passwordFile = args[0];

      if( args.length >= 2 )
	{
	  port = Integer.parseInt( args[1] );
	}

      try
	{
	  EncryptionStandard.getInstance().initializeEncryptionLibrary();

	  System.out.println( "Enter ticket server password:" );
	  char password[] = 
	    EncryptionStandard.getInstance().readPassword( System.in );
	  SecretKey tsKey = 
	    EncryptionStandard.getInstance().convertPassword( password );
	  Arrays.fill( password, '*' );

	  TicketServer s = new TicketServer( tsKey );
	  s.readPasswords( passwordFile );
	  s.open( DEFAULT_PORT );
	  System.out.println( "Read passwords from " + passwordFile );
	  System.out.println( "Listening for connections on port " + port );
	  s.run();
	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	}
    }
}
