/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package Celbercor;

import java.util.*;
import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;

/** These are functions that are useful for clients that wish to
 * authenticate themselves.  See <tt>SampleClient</tt> for an example
 * of how to use them.
 *
 * @see Celbercor.SampleClient */

public abstract class Utilities
{

  /** (Step 1) Try to contact the authentication server. <i>Note:</i>
   * This function currently spits out a lot of debugging information.
   *
   * @param host authentication server host
   * @param port authentication server port
   * @param endClientName your name (login, etc.)
   *
   * @return the reply if possible (a <tt>SealedObject</tt> containing
   * a <tt>AuthenticationReplyMessage</tt> encrypted with the client's
   * password key), or <tt>null</tt> if something went wrong */

  public static SealedObject
  contactAuthenticationServer( String host, int port, String endClientName )
    throws IOException, GeneralSecurityException, ClassNotFoundException
    {
      System.out.println( "Contacting authentication server " 
			  + host + ':' + port );
      Socket s = new Socket( host, port );
      ObjectOutput out = new ObjectOutputStream( s.getOutputStream() );
      System.out.println( "Sending authentication request" );
      out.writeObject( new AuthenticationRequestMessage( endClientName,
						  TicketServer.NAME ) );
      System.out.println( "Waiting for reply" );
      ObjectInput in = new ObjectInputStream( s.getInputStream() );
      SealedObject st = (SealedObject)in.readObject();
      if( st == null )
	{
	  System.out.println( "Authentication failed: "
			      + "unknown name or service" );
	  return null;
	}

      System.out.println( "Authentication message received" );
      return st;
    }

  /** (Step 3) Contact the ticket server and request a ticket for a
   * service.
   *
   * @param host the ticket server host
   * @param port the ticket server port
   * @param serviceName the name of the service you want a ticket for
   *
   * @param tst the ticket from the authentication server for
   * contacting the ticket server
   *
   * @param sealedTST the same ticket as <tt>tst</tt>, sealed under
   * the ticket server's private key.  (This is given to you by the
   * authentication server.)
   *
   * @return the ticket server's reply, or <tt>null</tt> if something
   * went wrong.
   * */

  public static TicketReplyMessage
  contactTicketServer( String host, int port,
		       String serviceName,
		       Ticket tst, SealedObject sealedTST )
    throws IOException, GeneralSecurityException, ClassNotFoundException
    {
      InetAddress addr = InetAddress.getLocalHost();
      SessionAuthenticator a = 
	new SessionAuthenticator( tst.getEndClientName(),
				  addr,
				  System.currentTimeMillis() );
      Cipher toTicketServer = 
	EncryptionStandard.getInstance().createCipher();
      toTicketServer.init( Cipher.ENCRYPT_MODE,
			   tst.getSessionKey() );
      SealedObject sealedA = new SealedObject( a, toTicketServer );

      System.out.println( "Contacting ticket server " 
			  + host + ':' + port );
      Socket s = new Socket( host, port );

      TicketRequestMessage trequest = 
	new TicketRequestMessage( sealedTST,
				  sealedA,
				  serviceName );
      System.out.println( "Sending ticket request" );
      ObjectOutput out = new ObjectOutputStream( s.getOutputStream() );
      out.writeObject( trequest );
      
      System.out.println( "Waiting for reply" );
      ObjectInput in = new ObjectInputStream( s.getInputStream() );
      SealedObject sealedReply = (SealedObject)in.readObject();
      TicketReplyMessage treply = 
	(TicketReplyMessage)sealedReply.getObject(tst.getSessionKey());

      return treply;
    }


  /** (Step 5) Contact the end service by sending a
   * <tt>ServiceContactMessage</tt> to the given output stream, and
   * get back a <tt>ServiceAcknowledgeMessage</tt>.
   *
   * @return the reply message plus input and output streams, or
   * <tt>null</tt> if something went wrong
   * */

  public static ServiceInfo
  contactService( String host, int port,
		  String endClientName,
		  String serviceName,
		  SecretKey sessionKey,
		  SealedObject sealedST )
    throws IOException, GeneralSecurityException, ClassNotFoundException
    {
      InetAddress addr = InetAddress.getLocalHost();
      SessionAuthenticator a = 
	new SessionAuthenticator( endClientName,
				  addr,
				  System.currentTimeMillis() );
      Cipher toTicketServer = 
	EncryptionStandard.getInstance().createCipher();
      toTicketServer.init( Cipher.ENCRYPT_MODE, sessionKey );
      SealedObject sealedA = new SealedObject( a, toTicketServer );
      ServiceContactMessage m = 
	new ServiceContactMessage( sealedST, sealedA, serviceName );

      Socket s = new Socket( host, port );
      ObjectOutput out = new ObjectOutputStream( s.getOutputStream() );

      System.out.println( "Sending contact message to service" );
      out.writeObject( m );

      System.out.println( "Waiting for acknowledgment" );
      ObjectInput in = new ObjectInputStream( s.getInputStream() );
      SealedObject sealedAck = (SealedObject)in.readObject();
      ServiceAcknowledgeMessage ack = 
	(ServiceAcknowledgeMessage)sealedAck.getObject( sessionKey );
      System.out.println( "Acknowledgment received" );

      return new ServiceInfo( ack, in, out );
    }

  /** This <tt>main()</tt> function just prints out the machine's IP
   * address.  It was used to see what
   * <tt>InetAddress.getLocalHost()</tt> returns. (There was a bug in
   * some of the ticket stuff related to comparing
   * "localhost/127.0.0.1" to a real IP address.)  */

  public static void main( String args[] )
    {
      try
	{
	  System.out.println( InetAddress.getLocalHost() );
	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	}
    }
}
