/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package ChatRoom.Celbercor;

import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;
import javax.crypto.*;
import Celbercor.*;
import ChatRoom.Security.*;
import ChatRoom.Tools.*;
import ChatRoom.Tools.Reader;
import ChatRoom.Tools.Writer;


/** This class is the implementaion of <tt>ClientAuthentication</tt>
 * for use with the Celbercor package.  To use this class, create an
 * instance of it, call <tt>initialize()</tt> on it, and use the
 * return value from <tt>sendVerification()</tt> to construct a
 * <tt>Client</tt> object.  A lot of the dirty work involved in that
 * can be found in the static methods of class <tt>SecureClient</tt>.
 *
 * @see ChatRoom.Security.SecureClient 
 * @see ChatRoom.Client.Client */

public class ClientAuthenticationImpl
  implements ClientAuthentication
{
  /**
   * @param name the login name of the client
   *
   * @param serviceName the name of the service to contact (normally
   * "chatroom")
   *
   * @param authHost host for the authentication server
   *
   * @param authPort port where the authentication server is listening
   *
   * @param ticketHost host for the ticket server
   *
   * @param ticketPort port where the ticket server is listening */

  public ClientAuthenticationImpl( String name, String serviceName,
				   String authHost, int authPort,
				   String ticketHost, int ticketPort )
    {
      myName = name;
      myServiceName = serviceName;
      myAuthHost = authHost;
      myAuthPort = authPort;
      myTicketHost = ticketHost;
      myTicketPort = ticketPort;
    }

  /*
   * <ul>
   * <li> <tt>celbercor.authentication.host</tt>.  
   * (<i>Default value: <tt>AuthenticationServer.DEFAULT_HOST</tt></i>)
   * <li> <tt>celbercor.authentication.port</tt>.
   * (<i>Default value: <tt>AuthenticationServer.DEFAULT_PORT</tt></i>)
   * <li> <tt>celbercor.ticket.host</tt>.  
   * (<i>Default value: <tt>TicketServer.DEFAULT_HOST</tt></i>)
   * <li> <tt>celbercor.ticket.port</tt>.
   * (<i>Default value: <tt>TicketServer.DEFAULT_PORT</tt></i>)
   * </ul>
   */

  /** Initialize this authentication object with some default values.
   * The service name is set to <tt>Constants.CHATROOM_NAME</tt>, and
   * the names and port of the authentication and ticket servers come
   * from the <a
   * href="../../Celbercor/package-summary.html#Properties">system
   * properties</a>, using a default value if the property is not set.
   *
   * @param name the login name to use
   * */

  public ClientAuthenticationImpl( String name )
    {
      this( name, Constants.CHATROOM_NAME,
	    System.getProperty( "celbercor.authentication.host",
				AuthenticationServer.DEFAULT_HOST ),
	    Integer.getInteger( "celbercor.authentication.port",
				AuthenticationServer.DEFAULT_PORT ).intValue(),
	    System.getProperty( "celbercor.ticket.host",
				TicketServer.DEFAULT_HOST ),
	    Integer.getInteger( "celbercor.ticket.port",
				TicketServer.DEFAULT_PORT ).intValue() );
    }

  /** Handle steps 1-4 of the protocol.  Contact the authentications
   * server (step 1), input a password, convert it to a key, decrypt
   * the reply (from step 2), contact the ticket server to get a key
   * for the chat room (step 3), and decode and verify the reply (from
   * step 4).
   *
   * @return true if all of that worked, false if it didn't */

  public boolean initialize()
    {
      try
	{
	  // Contact authentication server:

	  SealedObject so = 
	    Utilities.contactAuthenticationServer( myAuthHost, myAuthPort, 
						   myName );
	  
	  // Get password:

	  System.out.println( "Enter password:" );
	  //char password[] = { 'm', 'a', 'r', 's', 'e', 'l', 'l', 'e' };
	  char password[] = 
	    EncryptionStandard.getInstance().readPassword( System.in );
	  myKey = 
	    EncryptionStandard.getInstance().convertPassword( password );
	  Arrays.fill( password, '*' );

	  // Decode reply:

	  AuthenticationReplyMessage sreply = null;
	  Ticket tst = null;

	  sreply = (AuthenticationReplyMessage)so.getObject( myKey );
	  tst = sreply.getTicket();
	  String msg = tst.checkConsistency( myName,
					     InetAddress.getLocalHost(),
					     TicketServer.NAME,
					     System.currentTimeMillis() );
	  if( msg != null )
	    {
	      System.out.println( "Authentication server reply inconsistent" );
	      System.out.println( msg );
	      return false;
	    }

	  // Contact ticket server:

	  TicketReplyMessage treply =
	    Utilities.contactTicketServer( myTicketHost, myTicketPort, 
					   myServiceName,
					   tst, sreply.getSealedTicket() );
	  
	  
	  Ticket replyTicket = treply.getTicket();
	  msg = replyTicket.checkConsistency( myName,
					      InetAddress.getLocalHost(),
					      myServiceName,
					      System.currentTimeMillis() );
	  if( msg != null )
	    {
	      System.out.println( "Ticket server reply inconsistent" );
	      System.out.println( msg );
	      return false;
	    }

	  mySessionKey = treply.getSessionKey();
	  mySealedTicket = treply.getSealedTicket();

	}
      catch( Exception ex )
	{
	  // For debugging purposes:
	  ex.printStackTrace();
	  return false;
	}
      return true;
    }
  

  /** Handle steps 5-6 of the protocol.  Contact the service at the
   * given host and port (step 5), decode the reply (from step 6) and
   * create a reader and writer that can be used for further encrypted
   * communication with the service.  
   *
   * @return the reader and writer if successful, <tt>null</tt> if
   * something went wrong */

  public RWPair sendVerification( String host, int port )
    throws IOException, GeneralSecurityException, ClassNotFoundException
    {
      RWPair rval = null;

      // Contact service
      ServiceAcknowledgeMessage ack;
      ServiceInfo info =
	Utilities.contactService( host, port, myName, myServiceName,
				  mySessionKey, mySealedTicket );
      
      System.out.println( "Processing acknowledgment" );
      
      if( info == null )
	{
	  System.out.println( "Acknowledgment was null" );
	  return null;
	}
      
      ack = info.getMessage();
      String msg;
      msg = Ticket.checkTimes( ack.getTime(),
			       System.currentTimeMillis() );
      if( msg != null )
	{
	  System.out.println( "Acknowledgment was inconsistent" );
	  System.out.println( msg );
	  return null;
	}
      
      if( !ack.getEndClientName().equals(myName) )
	{
	  System.out.println( "End client name is wrong: "
			      + ack.getEndClientName()
			      + " should be "
			      + myName );
	  return null;
	}
      else if( !ack.getServiceName().equals(myServiceName) )
	{
	  System.out.println( "Service name is wrong: "
			      + ack.getServiceName()
			      + " should be "
			      + myName );
	  return null;
	}
      
      System.out.println( "Acknowledgment is consistent" );
      Reader r = RWFactory.createReader( mySessionKey, info.getInput() );
      Writer w = RWFactory.createWriter( mySessionKey, info.getOutput() );
      rval = new RWPair( r, w, myName );
      
      return rval;
    }

  private String myAuthHost;
  private int myAuthPort;
  private String myTicketHost;
  private int myTicketPort;
  private SecretKey myKey;
  private String myName;
  private String myServiceName;
  private SecretKey mySessionKey;
  private SealedObject mySealedTicket; // we send to chat room server
}
