/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package ChatRoom.Celbercor;

import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;
import javax.crypto.*;
import Celbercor.*;
import ChatRoom.Security.*;
import ChatRoom.Tools.*;
import ChatRoom.Tools.Reader;
import ChatRoom.Tools.Writer;
import ChatRoom.Tools.Message;

/** This class is the implementaion of <tt>ServerAuthentication</tt>
 * for use with the Celbercor package.  To use this class, create an
 * instance of it, call <tt>initialize()</tt>, and use it to construct
 * a <tt>SecureServer</tt> object.
 *
 * @see ChatRoom.Security.SecureServer */

public class ServerAuthenticationImpl
  implements ServerAuthentication
{

  /** @param name the name of this server, normally
   * <tt>Constants.CHATROOM_NAME</tt> */

  public ServerAuthenticationImpl( String name )
    {
      myName = name;
    }

  /**
   * Read the server's password and create an encryption key from it.
   */

  public synchronized boolean initialize()
    {
      char password[];
      try
	{
	  System.out.println( "Enter password:" );
	  password = 
	    EncryptionStandard.getInstance().readPassword( System.in );
	  myKey = 
	    EncryptionStandard.getInstance().convertPassword( password );
	  Arrays.fill( password, '*' );
	}
      catch( Exception ex )
	{
	  System.out.println( "Unexpected exception:" );
	  ex.printStackTrace();
	}
      return true;
    }

  /** Perform steps 5 and 6 of the protocol.  Attempt to read a
   * <tt>ServiceContactMessage</tt>, verify the information in it, and
   * send a reply.  The reply will be <tt>null</tt> if authentication
   * failed.
   *
   * @return a reader/writer pair that can communicate securely with
   * the client that contacted the server, or <tt>null</tt> if
   * authentication failed */

  public synchronized
  RWPair verifyClient( Socket s )
    {
      RWPair rval = null;

      try
	{
	  InetAddress clientAddr = s.getInetAddress();
	  SealedObject so;
	  Ticket t;
	  SessionAuthenticator a;
	  String msg;
	  SecretKey sessionKey;

	  System.out.println( "Reading service contact message" );
	  ObjectInput in = new ObjectInputStream( s.getInputStream() );
	  ServiceContactMessage m = 
	    (ServiceContactMessage)in.readObject();
	  so = m.getSealedTicket();
	  t = (Ticket)so.getObject( myKey );
	  sessionKey = t.getSessionKey();
	  so = m.getSealedAuthenticator();
	  a = (SessionAuthenticator)so.getObject( sessionKey );

	  msg = t.checkConsistency( a, myName, clientAddr,
				    System.currentTimeMillis() );

	  ObjectOutput out = new ObjectOutputStream( s.getOutputStream() );
	  if( msg != null )
	    {
	      System.out.println( msg );
	      out.writeObject( null );
	      rval = null;
	    }
	  else
	    {
	      System.out.println( "The message is consistent\n"
				  + "Formulating reply" );
	      myClientName = a.getEndClientName();
	      ServiceAcknowledgeMessage reply =
		new ServiceAcknowledgeMessage( t.getEndClientName(),
					       myName,
					       System.currentTimeMillis() );

	      Cipher toClient = 
		EncryptionStandard.getInstance().createCipher();
	      toClient.init( Cipher.ENCRYPT_MODE, sessionKey );
	      so = new SealedObject( reply, toClient );
	      System.out.println( "Sending reply" );
	      out.writeObject( so );
	      rval = new RWPair( createReader( in, sessionKey ),
				 createWriter( out, sessionKey ),
				 t.getEndClientName() );
	      System.out.println( "Reply sent" );
	    }
	}
      catch( IOException ex )
	{
	  System.out.println( "Connection failed:\n" );
	  ex.printStackTrace();
	  rval = null;
	}
      catch( Exception ex )
	{
	  System.out.println( "Unexpected exception:\n" );
	  ex.printStackTrace();
	  rval = null;
	}

      return rval;
    }


  private Reader createReader( ObjectInput in, SecretKey sessionKey )
    {
      return new CheckedReader( myClientName,
          RWFactory.createReader( sessionKey, in ) );
    }

  private Writer createWriter( ObjectOutput out, SecretKey sessionKey )
    {
      return RWFactory.createWriter( sessionKey, out );
    }

  private SecretKey myKey;
  private String myName;
  private String myClientName;
}
