/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package ChatRoom;

import java.io.*;
import java.awt.*;
import java.net.*;
import java.security.*;
import Celbercor.*;
import ChatRoom.Tools.*;
import ChatRoom.Client.*;
import ChatRoom.Security.*;
import ChatRoom.Celbercor.*;
import ChatRoom.GUI.*;

/** This <tt>main()</tt> function starts a chat room client that obeys
 * the Celbercor security protocol.  */

public abstract class CelbercorClient
{
  /** Start a client.  Attempt to contact the authentication server,
   * the ticket server, and finally the secure chat room server.
   * Along the way, ask for a password which is needed to decrypt the
   * authentication server's reply. 
   *
   * <p> The arguments to this function are:
   *
   * <pre>
   * java ChatRoom.CelbercorClient <i>name</i> [<i>chatHost</i> [<i>chatPort</i>]]
   * </pre>
   *
   * where <i>name</i> should be a login name, <i>chatHost</i> should
   * be the host where the chat server is running (default value is
   * {@link ChatRoomConstants#DEFAULT_HOST
   * <tt>ChatRoomConstants.DEFAULT_HOST</tt>}), and <i>chatPort</i> is
   * the port the server is listening to (default value is {@link
   * ChatRoomConstants#DEFAULT_PORT
   * <tt>ChatRoomConstants.DEFAULT_PORT</tt>}).  The host name and
   * port for the authentication and ticket servers are taken from
   * system properties documented in package {@link Celbercor
   * <tt>Celbercor</tt>}.
   * 
   * */

  public static void main( String args[] )
    {
      try
	{
	  EncryptionStandard.getInstance().initializeEncryptionLibrary();
	}
      catch( GeneralSecurityException ex )
	{
	  ex.printStackTrace();
	  System.exit(1);
	}

      String name;
      String host = ChatRoomConstants.DEFAULT_HOST;
      int port = ChatRoomConstants.DEFAULT_PORT;
      Client c = null;
      SimpleUI g;
      Frame frame;

      if( args.length < 1 )
	{
	  System.out.println( "Arguments: name [host [port]]" );
	  System.exit(2);
	}

      name = args[0];
      
      if( args.length >= 2 )
	{
	  host = args[1];
	}

      if( args.length >= 3 )
	{
	  port = Integer.parseInt( args[2] );
	}

      ClientAuthentication cAuth = new ClientAuthenticationImpl( name );

      if( !cAuth.initialize() )
	{
	  System.out.println( "Authentication failed" );
	  System.exit(3);
	}

      System.out.println( "Authentication suceeded" );

      try
	{
	  c = SecureClient.createViaInternet( host, port, 
					      cAuth );
	}
      catch( UnknownHostException ex )
	{
	  System.out.println( "Unable to contact chat room host " + host );
	  System.exit(4);
	}
      catch( IOException ex )
	{
	  System.out.println( "Could not contact chat room server" );
	  System.exit(5);
	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	  System.exit(6);
	}

      if( c == null )
	{
	  System.out.println( "Could not start client" );
	  System.exit(7);
	}

      System.out.println( "Contacted server, opening window:" );

      g = new SimpleUI( name, c, host, port );
      frame = new Frame( "Secure Chat:" + name );
      frame.add( g, "Center" );
      frame.pack();
      frame.addWindowListener( new ExitWhenClosed() );
      frame.setVisible( true );

      c.run();
      System.out.println( "Lost contact with the server." );
      System.exit(0);
      
    }
}
