/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package ChatRoom.FlamingSword;

import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;
import javax.crypto.*;
import ChatRoom.Security.*;
import ChatRoom.Tools.*;
import ChatRoom.Tools.Reader;
import ChatRoom.Tools.Writer;
import ChatRoom.Tools.Message;
import FlamingSword.*;

/** This class represents authentication objects that use the Flaming
 * Sword protocol.  To use it, create an instance of it, call
 * <tt>initialize()</tt>, and pass it to the constructor of a
 * <tt>SecureServer</tt> object.
 *
 * @see ChatRoom.Security.SecureServer  */

public class ServerAuthenticationImpl
  extends AgentBase
  implements ServerAuthentication
{
  /** @param fileName abbreviated name of files where the server's
   * public and private keys are held 
   *
   * @param authFileName abbreviated name of the file where the
   * authentication server's public key is held. */

  public ServerAuthenticationImpl( String fileName,
				   String authFileName )
    {
      myFileName = fileName;
      myAuthFileName = authFileName;
    }

  /** Load the user's public and private keys, and the authentication
   * server's public keys.  They keys are read from files specified by
   * the constructor.  */

  public synchronized boolean initialize()
    {
      try
	{
	  initialize( myFileName, myAuthFileName );
	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	  return false;
	}
      return true;
    }

  /** Perform steps 3 and 4 of the protocol.  Attempt to read a
   * <tt>ServiceContactMessage</tt>, and verify all the authenticaion
   * stuff.  Send a reply, (<tt>null</tt> if authentication failed),
   * and return a reader/writer pair.
   *
   * @return a reader/writer pair that can communicate securely with
   * the client that contacted the server, or <tt>null</tt> if
   * authentication failed */

  public synchronized
  RWPair verifyClient( Socket s )
    {
      RWPair rval = null;

      try
	{
	  System.out.println( "Reading request" );
	  ObjectOutput out = new ObjectOutputStream( s.getOutputStream() );
	  ObjectInput in = new ObjectInputStream( s.getInputStream() );
	  SealedObject srequest =
	    (SealedObject)in.readObject();
	  ServiceContactMessage request =
	    (ServiceContactMessage)srequest.getObject( getPrivateKey() );
	  request.decrypt( getAuthPublicKey() );
	  Ticket t = request.getTicket();
	  String msg;

	  msg = Ticket.checkTimes( t.getTime(), System.currentTimeMillis() );
	  if( msg != null )
	    {
	      System.out.println( msg );
	      out.writeObject( null );
	      return null;
	    }

	  PublicKey clientKey = t.getPublicKey();
	  Object nonce = request.getSealedNonce().getObject( clientKey );
	  if( !nonce.equals( t.getNonce() ) )
	    {
	      System.out.println( "Nonces mis-matched:\n"
				  + nonce
				  + "\nshould be\n"
				  + t.getNonce() );
	      out.writeObject( null );
	      return null;
	    }

	  // At this point, everything should be okay...
	  System.out.println( "Message was consistent, formulating reply" );

	  SealedObject resealedNonce = 
	    new SealedObject( (Serializable)nonce, getPrivateCipher() );
	  ServiceAcknowledgeMessage reply =
	    new ServiceAcknowledgeMessage( resealedNonce );
	  Cipher clientCipher = 
	    EncryptionStandard.getInstance().createCipher();
	  clientCipher.init( Cipher.ENCRYPT_MODE, clientKey );

	  SealedObject sealedReply =
	    new SealedObject( reply, clientCipher );
	  
	  out.writeObject( sealedReply );

	  clientCipher.init( Cipher.DECRYPT_MODE, clientKey );

	  Reader r =
	    new CheckedReader( t.getAgent(), 
	        new SignedSealedObjectReader( clientCipher, in, nonce ) );
	  Writer w = 
	    new SignedSealedObjectWriter( getPrivateCipher(), out, nonce );

	  rval = new RWPair( r, w, t.getAgent() );

	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	}

      return rval;
    }

  private String myFileName;
  private String myAuthFileName;
}
