/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package ChatRoom;

import java.io.*;
import java.awt.*;
import java.net.*;
import java.security.*;
import FlamingSword.*;
import ChatRoom.Tools.*;
import ChatRoom.Client.*;
import ChatRoom.Security.*;
import ChatRoom.FlamingSword.*;
import ChatRoom.GUI.*;

/** This <tt>main()</tt> function starts a GUI client which uses the
 * Flaming Sword authentication protocol.  */

public abstract class FlamingSwordClient
{
  /**
   * Start a Flaming Sword client.  It takes the following arguments:
   *
   * <pre>
   * java ChatRoom.FlamingSwordClient <i>name</i> <i>authKeyFile</i> [<i>host</i> [<i>port</i>]]
   * </pre>
   *
   * where: <i>name</i> is your login name and must also be the
   * abbreviated name of your key files (<tt>.pub</tt> and
   * <tt>.pri</tt> will be appended to this, and those files will be
   * read to get your public and private encryption keys);
   * <i>authKeyFile</i> is the abbreviated name of a file containing
   * the authentication server's public key (<tt>.pub</tt> will be
   * appended); <i>host</i> is the host where the chat server is
   * running (default value is
   * <tt>ChatRoomConstants.DEFAULT_HOST</tt>); <i>port</i> is the port
   * the server is listening to (default value is
   * <tt>ChatRoomConstants.DEFAULT_PORT</tt>). 
   *
   * <p> See also {@link FlamingSword <tt>FlamingSword</tt>} for a
   * list of system properties used to specify where the
   * authentication server is.*/

  public static void main( String args[] )
    {
      try
	{
	  EncryptionStandard.getInstance().initializeEncryptionLibrary();
	}
      catch( GeneralSecurityException ex )
	{
	  ex.printStackTrace();
	  System.exit(1);
	}

      String name;
      String authKeyFile;
      String host = ChatRoomConstants.DEFAULT_HOST;
      int port = ChatRoomConstants.DEFAULT_PORT;
      Client c = null;
      SimpleUI g;
      Frame frame;
      
      if( args.length < 2 )
	{
	  System.out.println( "Arguments: name authKeyFile [host [port]]" );
	  System.exit(2);
	}

      name = args[0];
      authKeyFile = args[1];

      if( args.length >= 3 )
	{
	  host = args[2];
	}

      if( args.length >= 4 )
	{
	  port = Integer.parseInt( args[3] );
	}

      ClientAuthentication cAuth = 
	new ClientAuthenticationImpl( name, authKeyFile );

      if( !cAuth.initialize() )
	{
	  System.out.println( "Authentication failed" );
	  System.exit(3);
	}

      System.out.println( "Authentication suceeded" );

      try
	{
	  c = SecureClient.createViaInternet( host, port, cAuth );
	}
      catch( UnknownHostException ex )
	{
	  System.out.println( "Unable to contact host " + host );
	  System.exit(4);
	}
      catch( IOException ex )
	{
	  System.out.println( "Could not contact chat room server" );
	  System.exit(5);
	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	  System.exit(6);
	}

      if( c == null )
	{
	  System.out.println( "Could not start client" );
	  System.exit(7);
	}

      System.out.println( "Contacted server, opening window:" );

      g = new SimpleUI( name, c, host, port );
      frame = new Frame( "Secure Chat:" + name );
      frame.add( g, "Center" );
      frame.pack();
      frame.addWindowListener( new ExitWhenClosed() );
      frame.setVisible( true );

      c.run();
      System.out.println( "Lost contact with the server." );
      System.exit(0);
      
    }
}
