/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package ChatRoom.Tools;

import java.net.*;
import java.io.*;

/** This class sends messages down a DatagramSocket.  It truncates
 * messages that are longer than the given packet size.  When it does
 * truncate them, it preserves as much of the name as possible and
 * chops off the end of the message. */

public class DatagramWriter
  implements Writer
{
  /** @param s a socket to write to
   *
   * @param a address to send to
   *
   * @param port port to send to
   *
   * @param packetSize how big the biggest packets will be.
   */

  public DatagramWriter( DatagramSocket s, InetAddress a, 
			 int port, int packetSize )
    {
      mySocket = s;
      myAddress = a;
      myPort = port;
      mySize = packetSize;

      if(packetSize < 7 )
	{
	  throw 
	    new IllegalArgumentException( "packetSize must be at least 7" );
	}
    }

  /** Deliver the message. Throw an IOException if it fails. */

  public void deliverMessage( Message m )
    throws IOException
    {
      Message m2 = truncateIfNeeded( m );
      ByteArrayOutputStream b = new ByteArrayOutputStream( mySize );
      DataOutput out = new DataOutputStream( b );
      DatagramPacket p;
      byte a[];

      DataOutputWriter.writeMessage( out, m );
      a = b.toByteArray();
      p = new DatagramPacket( a, a.length, myAddress, myPort );
      mySocket.send( p );
    }

  private Message truncateIfNeeded( Message m )
    {
      Message m2;

      int nameSize = 1 + m.getName().length();
      int textSize = 4 + m.getText().length();
      int messageSize =  nameSize + textSize;
      int n;

      if( messageSize > mySize )
	{
	  if( nameSize > mySize - 6 )
	    {
	      // 6 = 1 + 4 + 1, byte for namelen,
	      // 4 bytes for textlen,
	      // and at least 1 byte of text.
	      n = mySize - 6; 
	      
	      m2 = new Message( m.getName().substring( 0, n ),
				m.getText() );
	      m2 = truncateIfNeeded( m2 );
	    }
	  else
	    {
	      // 5 = 4 + 1, 4 bytes for textlen and at least 1 byte of
	      // text
	      n = mySize - nameSize - 5;

	      m2 = new Message( m.getName(),
				m.getText().substring( 0, n ) );
	    }
	  
	}
      else
	{
	  m2 = m;
	}

      return m2;

    }


  private DatagramSocket mySocket;
  private InetAddress myAddress;
  private int myPort;
  private int mySize;
}
