package FlamingSword;

import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;
import javax.crypto.*;

/** This class contains some static functions that make it cleaner for
 * clients to use the Flaming Sword protocol.  */

public abstract class Utilities
{
  /** Handle steps 1-2 of the protocol.  Contact the given
   * authentication server and attempt to get tickets which will allow
   * the client to talk to the desired end service.
   *
   * @param endClient this client's name
   *
   * @param endService the name of the service that the client
   * eventually wants to talk to
   *
   * @param host hostname for the authentication server
   *
   * @param port port number for the authentication server
   *
   * @param serverKey the authentication server's public key
   *
   * @param clientKey the client's private key
   *
   * @return the server's reply, fully decrypted, or <tt>null</tt> if
   * something went wrong
   * */

  public static AuthenticationReplyMessage 
  contactAuthenticationServer( String endClient, String endService,
			       String host, int port,
			       PublicKey authServerKey,
			       PrivateKey clientKey )
    throws IOException, GeneralSecurityException
    {
      AuthenticationReplyMessage reply = null;
      Socket s = null;
      ObjectOutput out = null;
      ObjectInput in = null;

      System.out.println( "Connecting to authentication server "
			  + host + ':' + port );

      try
	{
	  s = new Socket( host, port );
	  out = new ObjectOutputStream( s.getOutputStream() );
	  in = new ObjectInputStream( s.getInputStream() );
	}
      catch( IOException ex )
	{
	  System.out.println( "Unable to contact the server:" );
	  ex.printStackTrace();
	  return null;
	}

      try
	{
	  System.out.println( "Sending request" );
	  
	  AuthenticationRequestMessage request =
	    new AuthenticationRequestMessage( endClient, endService );
	  
	  out.writeObject( request );
	  
	  System.out.println( "Waiting for reply" );
	  SealedObject so = (SealedObject)in.readObject();
	  reply = (AuthenticationReplyMessage)so.getObject( clientKey );
	  reply.decrypt( authServerKey );

	  String msg = 
	    reply.getServiceTicket()
	    .checkConsistency( endService, System.currentTimeMillis() );
	  if( msg != null )
	    {
	      System.out.println( "Service ticket was inconsistent:\n"
			      + msg );
	      return null;
	    }

	  msg =
	    reply.getClientTicket()
	    .checkConsistency( endClient, System.currentTimeMillis() );
	  if( msg != null )
	    {
	      System.out.println( "Client ticket was inconsistent:\n"
			      + msg );
	      return null;
	    }
	  
	}
      catch( Exception ex )
	{
	  System.out.println( "Connection failed" );
	  ex.printStackTrace();
	  return null;
	}
      
      return reply;
    }

  /** Handle steps 3-4 of the protocol.  Contact the desired service,
   * and sent it a message containing a sealed ticket and a copy of
   * the nonce sealed under the client's key.
   * 
   * @param sealedTicket ticket sealed under the authentication
   * server's private key
   *
   * @param sealedNonce the nonce sealed under the client's private key
   *
   * @param host the host for the service
   *
   * @param port the port where the service is listening
   *
   * @param clientKey the client's private key
   *
   * @param serviceKey the service's public key
   *
   * @return the service's reply and i/o streams that can talk to the
   * service if it accepted our authentication, <tt>null</tt> if
   * something went wrong.
   * */

  public static ServiceInfo
  contactService( SealedObject sealedTicket,
		  Object nonce,
		  String host, int port,
		  PrivateKey clientKey, PublicKey serviceKey )
    {
      ServiceAcknowledgeMessage reply = null;
      Socket s = null;
      ObjectOutput out = null;
      ObjectInput in = null;

      try
	{
	  s = new Socket( host, port );
	  out = new ObjectOutputStream( s.getOutputStream() );
	  in = new ObjectInputStream( s.getInputStream() );
	}
      catch( IOException ex )
	{
	  System.out.println( "Unable to contact the server:" );
	  ex.printStackTrace();
	  return null;
	}

      try
	{
	  System.out.println( "Sending contact message" );

	  Cipher clientPrivate = 
	    EncryptionStandard.getInstance().createCipher();
	  clientPrivate.init( Cipher.ENCRYPT_MODE, clientKey );

	  SealedObject sealedNonce = 
	    new SealedObject( (Serializable)nonce, clientPrivate );
	  ServiceContactMessage request =
	    new ServiceContactMessage( sealedTicket, sealedNonce );

	  Cipher servicePublic =
	    EncryptionStandard.getInstance().createCipher();
	  servicePublic.init( Cipher.ENCRYPT_MODE, serviceKey );

	  SealedObject sealedRequest = 
	    new SealedObject( request, servicePublic );
	  out.writeObject( sealedRequest );

	  System.out.println( "Reading reply" );
	  SealedObject sreply = (SealedObject)in.readObject();
	  reply = (ServiceAcknowledgeMessage)sreply.getObject( clientKey );

	  reply.decrypt( serviceKey );
	  if( reply.getNonce().equals( nonce ) )
	    {
	      System.out.println( "Reply was consistent" );
	    }
	  else
	    {
	      System.out.println( "Reply was inconsistent" );
	      reply = null;
	    }

	}
      catch( Exception ex )
	{
	  System.out.println( "Connection failed" );
	  ex.printStackTrace();
	  reply = null;
	}

      if( reply == null )
	return null;
      else
	return new ServiceInfo( reply, in, out );
    }

}
